#!/usr/bin/env python
import os
import sys
import urllib
from xml.dom import minidom
from alunn.log import log_entry
import gettext
import locale

locale.setlocale(locale.LC_ALL, '')
gettext.install('alunn')

class NewsParser:
    news_id_file = os.path.expanduser('~/.alunn.news')
    current_news_id = ''

    def get_news(self):
        titles = []
        urls = []

        try:
            news_fp = open(self.news_id_file)
            try:
                old_news_id = news_fp.read().split("\n")[0]
            finally:
                news_fp.close()
        except IOError:
            old_news_id = ''

        try:
            news_feed = urllib.urlopen('http://www.archlinux.org/feeds/news/')
            try:
                news_xml = minidom.parse(news_feed)
            finally:
                news_feed.close()
        except IOError:
            log_entry(_("Failed downloading news feed"), False, True)
            return {'titles':[], 'urls':[]}

        news_items = news_xml.getElementsByTagName('item')
        self.current_news_id = news_items[0].getElementsByTagName('guid')[0].firstChild.data

        if self.current_news_id != old_news_id:
            for item in news_items:
                if old_news_id == item.getElementsByTagName('guid')[0].firstChild.data:
                    break
                titles.append(item.getElementsByTagName('title')[0].firstChild.data.strip())
                urls.append(item.getElementsByTagName('link')[0].firstChild.data)
        return {'titles':titles, 'urls':urls}

    def write_news(self):
        try:
            news_fp = open(self.news_id_file, 'w')
            try:
                news_fp.write(self.current_news_id)
            finally:
                news_fp.close()
        except IOError:
            sys.exit(_("ERROR") + ": " + _("writing to, or creating %s") % self.news_id_file)
        

    def __init__(self):
        pass
