# -*- coding: utf-8 -*-
import gtk
import gobject
import gnomevfs
try:
    import dbus
except:
    pass
import os
import sys
import time
from alunn.log import *
import gettext
import locale

locale.setlocale(locale.LC_ALL, '')
gettext.install('alunn')

class NotifyIcon:
    status = {'updates':False, 'news':False}
    tooltip = {
        'updates': _("Updates available!"),
        'news':    _("Unread news!"),
        'both':   _("Updates available and unread news!")
        }
    urls = []

    pacman_icon_file = "/usr/share/pixmaps/alunn-pacman.png"
    arch_icon_file = "/usr/share/pixmaps/alunn-arch.png"
    updatecommand = None
    webreader = None
    news = None
    updates_last_time = []
    news_last_time = []

    try:
        system_bus = dbus.SystemBus()
    except (NameError, dbus.exceptions.DBusException):
        pass
    try:
        session_bus = dbus.SessionBus()
    except (NameError, dbus.exceptions.DBusException):
        pass

    # Check if system support notifications
    try:
        if check_service('org.freedesktop.Notifications', session_bus) or dbus.Interface(session_bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus'), 'org.freedesktop.DBus').StartServiceByName('org.freedesktop.Notifications', dbus.UInt32(0)) == 1:
            notification = dbus.Interface(session_bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications'), 'org.freedesktop.Notifications')
            notificationid = 0
        else:
            notification = None
    except NameError:
        notification = None
    except dbus.exceptions.DBusException:
        notification = None
    if not notification:
        log_entry(_("System is not able to show notifications."), True, True)

    tray = gtk.status_icon_new_from_stock(gtk.STOCK_DIALOG_INFO)
    tray.set_visible(False)

    menu_update = gtk.ImageMenuItem(_("Update packages"))
    menu_news = gtk.ImageMenuItem(_("Read news"))

    def show_menu(self, status_icon, button, activate_time, menu):
        menu.popup(None, None, gtk.status_icon_position_menu, button, activate_time, status_icon)
        if self.notification:
            self.notification.CloseNotification(self.notificationid)
    
    def launch_update(self, widget=None):
        self.status['updates'] = False
        os.system(self.updatecommand)
        self.menu_update.hide()
        self.toggle_icon()

    def read_news(self, widget=None):
        self.status['news'] = False
        self.menu_news.hide()
        self.toggle_icon()
        self.news.write_news()
        os.system(self.webreader + " " + " ".join(self.urls) + " &")

    def launch_configuration(self, widget):
        os.system("alunnconf &")
        
    def show_about(self, widget):
        aboutdialog = gtk.AboutDialog()
        icon_pacman = gtk.image_new_from_icon_name('info', gtk.ICON_SIZE_SMALL_TOOLBAR)
        aboutdialog.set_name("Alunn")
        aboutdialog.set_version("0.7.16")
        aboutdialog.set_copyright(_("Copyright") + " © 2006-2009, Mathias Nedrebø")
        aboutdialog.set_website("http://www.nedrebo.org/code/alunn/")
        aboutdialog.set_website_label(_("Alunn Homepage"))
        aboutdialog.set_logo(gtk.gdk.pixbuf_new_from_file(self.pacman_icon_file));
        aboutdialog.set_comments(_("Arch Linux updates and news notification."))
        aboutdialog.set_license("""Alunn version 0.7.16
Copyright (C) 2006  Mathias Nedrebø, mathias@nedrebo.org 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public
License along with this program; if not, write to the Free
Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
""")
        aboutdialog.set_authors(["Mathias Nedrebø <mathias@nedrebo.org>"])
        aboutdialog.set_translator_credits(_("translator-credits")) #. Translate this string into your name and e-mail address
        aboutdialog.connect ("response", lambda d, r: d.destroy())
        aboutdialog.show()

    def show_icon(self, widget=None):
        self.tray.set_visible(True)

    def hide_icon(self, widget=None):
        self.tray.set_visible(False)

    def toggle_icon(self, widget=None):
        self.set_tooltip()
        if self.status['updates'] or self.status['news']:
            self.show_icon()
        else:
            self.hide_icon()

    def update_button(self, n, action):
        self.launch_update()

    def set_tooltip(self):
        if self.status['updates'] and self.status['news']:
            current_tip = self.tooltip['both']
        elif self.status['updates']:
            current_tip = self.tooltip['updates']
        elif self.status['news']:
            current_tip = self.tooltip['news']
        else:
            current_tip = _("This icon should not be here or this tooltip should at least say something else :(")
        self.tray.set_tooltip(current_tip)

    def show_notify_icon(self, title, message, type, urls=None):
        if message != self.news_last_time and message != self.updates_last_time:
            log_entry(type.capitalize() + ": " + message.replace("\n", "; "), False)
            if type == _("updates"):
                self.updates_last_time = message
                self.menu_update.show()
                self.status['updates'] = True
                icon = self.pacman_icon_file
            else:
                self.news_last_time = message
                self.urls = urls
                self.menu_news.show()
                self.status['news'] = True
                icon = self.arch_icon_file
            # Workaround for bug(?) in libnotify
            message_old = message
            message = ''
            for char in message_old:
                if char == '&':
                    message += 'and'
                else:
                    message += char
            self.toggle_icon()
            return (icon, title, message)
        else:
            return False

    def show_notify_balloon(self, args):
        if args and self.notification:
            tray_geometry = self.tray.get_geometry()[1]
            x = tray_geometry.x + tray_geometry.width / 2
            y = tray_geometry.y + tray_geometry.height / 2
            self.notificationid = self.notification.Notify("Update manager", 0, args[0], args[1], args[2], "", {'x':x, 'y':y}, 16000) 

    def __init__(self, updatecommand, webreader, news):
        self.updatecommand = updatecommand
        self.webreader = webreader
        self.news = news
        menu = gtk.Menu()
        
        self.menu_update.connect("activate", self.launch_update)
        self.menu_update.connect("activate", self.toggle_icon)
        icon_update = gtk.image_new_from_icon_name(gtk.STOCK_REFRESH, gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.menu_update.set_image(icon_update)
        menu.append(self.menu_update)
        
        self.menu_news.connect("activate", self.read_news)
        self.menu_news.connect("activate", self.toggle_icon)
        icon_news = gtk.image_new_from_icon_name(gtk.STOCK_FIND, gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.menu_news.set_image(icon_news)
        menu.append(self.menu_news)
        
        menu_hide = gtk.ImageMenuItem(_("Hide icon"))
        menu_hide.connect("activate", self.hide_icon)
        icon_hide = gtk.image_new_from_icon_name(gtk.STOCK_REMOVE, gtk.ICON_SIZE_SMALL_TOOLBAR)
        menu_hide.set_image(icon_hide)
        menu.append(menu_hide)
        menu_hide.show()
    
        # make configure menu if alunnconf is installed
        alunnconf = False
        for dir in os.environ['PATH'].split(':'):
            if os.path.isdir(dir):
                if 'alunnconf' in os.listdir(dir):
                    menu_configure = gtk.ImageMenuItem(_("Preferences"))
                    menu_configure.connect("activate", self.launch_configuration)
                    icon_configure = gtk.image_new_from_icon_name(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_SMALL_TOOLBAR)
                    menu_configure.set_image(icon_configure)
                    menu.append(menu_configure)
                    menu_configure.show()

        menu_separator = gtk.SeparatorMenuItem()
        menu.append(menu_separator)
        menu_separator.show()
    
        menu_about = gtk.ImageMenuItem(_("About"))
        menu_about.connect("activate", self.show_about)
        icon_about = gtk.image_new_from_icon_name(gtk.STOCK_ABOUT, gtk.ICON_SIZE_SMALL_TOOLBAR)
        menu_about.set_image(icon_about)
        menu.append(menu_about)
        menu_about.show()
    
        menu.show()

        self.tray.connect('popup-menu', self.show_menu, menu)
        self.set_tooltip()

        gtk.window_set_default_icon_from_file(self.pacman_icon_file);

        # Configure About Dialog
        gtk.about_dialog_set_email_hook(lambda a, m: gnomevfs.url_show("mailto:%s" % m))
        gtk.about_dialog_set_url_hook(lambda a, l: gnomevfs.url_show(l))
