#!/usr/bin/env python

import os
import string
import urllib
import tarfile
from alunn.log import *
import gettext
import locale

locale.setlocale(locale.LC_ALL, '')
gettext.install('alunn')

class UpdateChecker:
    pacman_conf_file = '/etc/pacman.conf'
    pacman_dir = '/etc/pacman.d/'
    transtable = string.maketrans(string.ascii_lowercase,string.ascii_lowercase[::-1])

    def check(self):
        try:
            pacman_conf_fp = open(self.pacman_conf_file)
            try:
                pacman_conf_list = pacman_conf_fp.read().split("\n")
                ignore_pkg = []
                for elm in pacman_conf_list:
                    if elm.split("=")[0].strip() in ["IgnorePkg"]:
                        ignore_pkg.extend([ e.strip() for e in elm.split("=")[1].split(" ") if e.strip() != ''])
                repositories =  []
                # Read bottom to top, to get the right order of the repositories
                for i in range(len(pacman_conf_list) - 1, 0, -1):
                    if len(pacman_conf_list[i].strip()) > 0 and pacman_conf_list[i].strip()[0] == '[' and not 'options' in pacman_conf_list[i]:
                        repositories.append(pacman_conf_list[i].strip()[1:-1])
                        while len(pacman_conf_list[i].strip().split("=")) == 0 or pacman_conf_list[i].strip().split("=")[0].strip() not in ["Server", "Include"]:
                            i = i + 1
                        repositories.append(pacman_conf_list[i].split("=")[0].strip())
                        repositories.append(pacman_conf_list[i].split("=")[1].strip())
            finally:
                pacman_conf_fp.close()
        except IOError:
            sys.exit(_("ERROR") + ": " + _("problem opening, reading, parsing or closing %s") % self.pacman_conf_file)

        packages = {}
        db_file_downloaded = False
        for i in range(0, len(repositories), 3):
            if repositories[i + 1] == 'Server':
                try:
                    db_file = urllib.urlretrieve(repositories[i + 2].replace('$repo', repositories[i]) + "/" + repositories[i] + '.db.tar.gz')[0]
                    db_file_downloaded = True
                except IOError:
                    log_entry('[' + repositories[i] + '] ' + _("failed to download:") + ' ' + repositories[i + 2] + "/" + repositories[i] + '.db.tar.gz', True, True)
            else: # Include
                try:
                    pacman_server_fp = open(repositories[i + 2])
                    try:
                        pacman_server_list = pacman_server_fp.read().split("\n")
                        servers =  [line.split("=")[1].strip().replace('$repo', repositories[i]) for line in pacman_server_list if line.split("=")[0].strip() == 'Server']
                    finally:
                        pacman_conf_fp.close()
                except IOError:
                    sys.exit(_("ERROR") + ": " + (_("problem opening, reading or closing %s") + "%s") % (self.pacman_dir, repositories[i]))
                for server in servers:
                    try:
                        db_file = urllib.urlretrieve(server + '/' + repositories[i] + '.db.tar.gz')[0]
                        db_file_downloaded = True
                        break
                    except IOError:
                        log_entry('[' +  repositories[i] + '] ' + _("Failed to download:") + ' ' + server + "/" + repositories[i] + '.db.tar.gz, trying next server on list.', True, True)
                        if server == servers[-1]:
                            log_entry(_("no more servers on list, giving up on [%s].") % repositories[i], False, False)
            
            if db_file_downloaded:
                try:
                    repo_db = tarfile.open(db_file, 'r:gz')
                    try:
                        for elm in repo_db:
                            if elm.name[-4:] == 'desc':
                                package_desc_fp = repo_db.extractfile(elm.name)
                                package_desc_list = package_desc_fp.read().split("\n")
                                packages[package_desc_list[package_desc_list.index("%NAME%") + 1]] = package_desc_list[package_desc_list.index("%VERSION%") + 1]
                                package_desc_fp.close()
                    finally:
                        repo_db.close()
                except IOError:
                    log_entry(_("Failed reading database file for repository %s") % repositories[i], False, True)
                
        # remove all tmp files
        urllib.urlcleanup()

        packages_current_fp = os.popen("pacman -Q")
        packages_current_list = packages_current_fp.read().split("\n")
        packages_current = dict([elm.split(" ") for elm in packages_current_list if elm.strip() != ''])
        packages_current_fp.close()
    
        updates = []
        for package, version in packages_current.items():
            if package not in ignore_pkg:
                try:
                    v_curr = version.split('-')
                    v_repo = packages[package].split('-')
                    v_curr = v_curr[0].split('.') + [v_curr[-1]]
                    v_repo = v_repo[0].split('.') + [v_repo[-1]]
                    while len(v_curr) < len(v_repo):
                        v_curr.insert(-1, 0)
                    while len(v_repo) < len(v_curr):
                        v_repo.insert(-1, 0)
                    for i in range(len(v_curr)):
                        v_curr[i] = string.translate(v_curr[i], self.transtable)
                        v_repo[i] = string.translate(v_repo[i], self.transtable)
                        len_ci = len(v_curr[i])
                        len_ri = len(v_repo[i])
                        if len_ci > len_ri:
                            if v_curr[i].isdigit() and v_repo[i].isdigit():
                                v_repo[i] = '0' * (len_ci - len_ri) + v_repo[i]
                            else:
                                v_repo[i] = v_repo[i] + 'z' * (len_ci - len_ri)
                        if len_ri > len_ci:
                            if v_curr[i].isdigit() and v_repo[i].isdigit():
                                v_curr[i] = '0' * (len_ri - len_ci) + v_curr[i]
                            else:
                                v_curr[i] = v_curr[i] + 'z' * (len_ri - len_ci)
                    old = False
                    for i in range(len(v_curr)):
                        if v_repo[i] >= v_curr[i]:
                            if v_repo[i] > v_curr[i]:
                                old = True
                        else:
                            break
                    if old:
                        updates.append(package)
                except:
                    pass

        return updates or ['None']

    def __init__(self):
        pass
